import { GetRoleConfig, IPermission, IRole, InferPermissions, InferResources, RoleAddResult, RoleRemoveResult, RoleUpdateResult, TRoleOptions, addOptions, removeOptions, updateOptions } from '../types';

/**
 * `Role` class
 */
declare class Role<T extends TRoleOptions> implements IRole<T> {
    name: T['name'];
    description?: string;
    meta?: Record<string, any>;
    config: T['config'];
    resources: InferResources<T['config']>[];
    permissions: InferPermissions<T['config']>;
    constructor(options: T);
    private getDefaultPermission;
    private _init;
    getPermissions(): InferPermissions<T["config"]>;
    getResources(): InferResources<T['config']>[];
    add<S extends string, P extends IPermission>(options: addOptions<S, P>): RoleAddResult<T, S, P>;
    remove<S extends keyof this['permissions'] & (string & {})>(options: removeOptions<S>): RoleRemoveResult<T, S>;
    update<S extends keyof this['permissions'] & (string & {}), P extends IPermission>(options: updateOptions<S, P>): RoleUpdateResult<T, S, P>;
    can<S extends keyof this['permissions'], P extends keyof this['permissions'][S]>(resource: S, permission: P): boolean;
    cannot<S extends keyof this['permissions'], P extends keyof this['permissions'][S]>(resource: S, permission: P): boolean;
    canAny<S extends keyof this['permissions']>(resource: S, permissions: (keyof this['permissions'][S])[]): boolean;
    canAll<S extends keyof this['permissions']>(resource: S, permissions: (keyof this['permissions'][S])[]): boolean;
    clone(): Role<T>;
    /**
     * Converts a role to a JSON string
     * You can optionally pass in a transform function to further transform the JSON string
     */
    toJSON(): string;
    toJSON<R>(transform?: (data: string) => R): R;
    /**
     * Converts a role to an object
     * You can optionally pass in a transform function to further transform the object
     */
    toObject(): GetRoleConfig<T>;
    toObject<R>(transform?: (data: GetRoleConfig<T>) => R): R;
    /**
     *  Creates a role from a JSON string
     */
    static fromJSON<T extends TRoleOptions>(json: string): Role<{
        config: T["config"];
        description: T["description"];
        meta: T["meta"];
        name: T["name"];
    }>;
    /**
     * Creates a role from object
     */
    static fromObject<T extends TRoleOptions>(object: T): Role<{
        config: T["config"];
        description: T["description"];
        meta: T["meta"];
        name: T["name"];
    }>;
    /**
     * This method can be used when you want to create a role from an unknown data source
     * @example
     *
     * ```ts
     * // create a role from an encrypted string using aes-256-cbc algorithm
     * import crypto from 'crypto';
     *
     * const key = 'somekey';
     * const iv = 'someiv';
     * const algorithm = 'aes-256-cbc';
     * const key_buffer = Buffer.from(key);
     * const iv_buffer = Buffer.from(iv);
     *
     * const encrypted_role_json_str = 'some encrypted data';
     *
     * const decrypt = (data: string) => {
     *    const decipher = crypto.createDecipheriv(algorithm, key_buffer, iv_buffer);
     *    let decrypted = decipher.update(data, 'hex', 'utf8');
     *    decrypted += decipher.final('utf8');
     *    return decrypted;
     * };
     * const role = Role.from(encrypted_role_json_str, (data) => {
     *    const decrypted = decrypt(data); // decrypt the data
     *    return Role.fromJSON(decrypted).toObject(); // convert the decrypted data to a role object and return it
     * });
     * ```
     */
    static from<T extends TRoleOptions, D>(data: D, transform: (data: D) => GetRoleConfig<T>): Role<GetRoleConfig<T>>;
    /**
     * Validates the role object
     */
    static validate(object: any, cb?: (result: boolean, errors: string[]) => void): string[];
    static validate(object: any): string[];
}
export default Role;
